package cn.winhc.elasticsearch.plugin;

import cn.winhc.elasticsearch.plugin.analyzer.WinhcAnalyzerProvider;
import cn.winhc.elasticsearch.plugin.handle.Text2Tokenizer;
import cn.winhc.elasticsearch.plugin.tokenizer.WinhcTokenizerFactory;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.elasticsearch.index.analysis.AnalyzerProvider;
import org.elasticsearch.index.analysis.TokenizerFactory;
import org.elasticsearch.indices.analysis.AnalysisModule;
import org.elasticsearch.plugins.AnalysisPlugin;
import org.elasticsearch.plugins.Plugin;

import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/11/8 09:56
 */
public class WinhcAnalysisPlugin extends Plugin implements AnalysisPlugin {
    public static final String PLUGIN_NAME = "winhc-plugin";
    private static final Logger logger = LogManager.getLogger(WinhcAnalysisPlugin.class);

    public WinhcAnalysisPlugin() {
        super();
        logger.info(PLUGIN_NAME + " installed into elasticsearch");
    }

    public static Set<Text2Tokenizer> getText2Token() {
        return WinhcTokenizerClazzNameConfiguration.clazzName.stream().map(c -> {
            try {
                return ((Text2Tokenizer) Class.forName(c).newInstance());
            } catch (Exception e) {
                logger.error("new instance error: {} \n{}", c, e);
                return null;
            }
        }).filter(Objects::nonNull).collect(Collectors.toSet());
    }

    @Override
    public Map<String, AnalysisModule.AnalysisProvider<TokenizerFactory>> getTokenizers() {
        Set<Text2Tokenizer> text2Token = getText2Token();
        Map<String, AnalysisModule.AnalysisProvider<TokenizerFactory>> collect = text2Token.stream().collect(Collectors.toMap(o -> "winhc_" + o.getTokenizerName(), o -> (indexSettings, environment, name, settings) -> WinhcTokenizerFactory.getTokenizerFactory(indexSettings, environment, name, settings, o)));
        logger.info("register tokenizers : {}", collect.keySet());
        return collect;
    }

    @Override
    public Map<String, AnalysisModule.AnalysisProvider<AnalyzerProvider<? extends Analyzer>>> getAnalyzers() {
        Set<Text2Tokenizer> text2Token = getText2Token();
        Map<String, AnalysisModule.AnalysisProvider<AnalyzerProvider<? extends Analyzer>>> collect = text2Token.stream().collect(Collectors.toMap(o -> "winhc_" + o.getTokenizerName(), o -> (AnalysisModule.AnalysisProvider) (indexSettings, environment, name, settings) -> WinhcAnalyzerProvider.getInstance(indexSettings, environment, name, settings, o)));
        logger.info("register analyzer : {}", collect.keySet());
        return collect;
    }
}
