package cn.winhc.elasticsearch.plugin.handle;

import cn.winhc.elasticsearch.plugin.entity.Term;
import cn.winhc.elasticsearch.plugin.util.DomainPostfixUtils;
import cn.winhc.elasticsearch.plugin.util.DomainUtils;
import cn.winhc.elasticsearch.plugin.util.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author: XuJiakai
 * 2021/11/8 09:07
 */
public class UrlTextToTokenizer implements Text2Tokenizer {
    private String tokenizerName = "url";

    private static final Pattern urlPattern = Pattern.compile("((http://)|(https://))?(www\\.)?([0-9a-zA-Z\\-_\\u4e00-\\u9fa5]+(\\.[a-zA-Z\\u4e00-\\u9fa5]+){1,2}).*");

    @Override
    public String getTokenizerName() {
        return tokenizerName;
    }


    private static List<Term> domain2Tokenizer(String orgUrl, String domain) {
        if (StringUtils.isBlank(domain)) {
            return Arrays.asList();
        }

        ArrayList<Term> list = new ArrayList<>();

        Matcher matcher = urlPattern.matcher(domain);
        if (matcher.matches()) {
            String url = matcher.group(5);
            String trimUrl = DomainPostfixUtils.trimDomainPostfix(url);

            int urlIndex = orgUrl.indexOf(url);
            int trimUrlIndex = orgUrl.indexOf(trimUrl);
            list.add(
                    Term.of(url, urlIndex, urlIndex + url.length(), "url")
            );
            list.add(
                    Term.of(trimUrl, trimUrlIndex, trimUrlIndex + trimUrl.length(), "trim_url")
            );
            if (trimUrl.contains(".")) {
                for (String s : trimUrl.split("\\.")) {
                    int subUrlIndex = orgUrl.indexOf(s);
                    list.add(
                            Term.of(s, subUrlIndex, subUrlIndex + s.length(), "sub_url")
                    );
                }
            }
        }
        if (!orgUrl.equals(domain)) {
            int i = orgUrl.indexOf(domain);
            list.add(
                    Term.of(domain, i, i + domain.length(), "org_url")
            );
        }
        return list.stream().distinct().collect(Collectors.toList());
    }

    @Override
    public List<Term> getTextTokenizer(String line) {
        List<Term> collect = DomainUtils.getDomain(line).stream()
                .flatMap(r -> {
                    List<Term> terms = domain2Tokenizer(line, r);
                    return terms.stream();
                }).collect(Collectors.toList());
        if (collect.isEmpty()) {
            return Collections.singletonList(Term.of(line, 0, line.length(), "keyword"));
        } else {
            return collect;
        }
    }

    public static void main(String[] args) {
        UrlTextToTokenizer urlTextToTokenizer = new UrlTextToTokenizer();
        List<Term> textTokenizer = urlTextToTokenizer.getTextTokenizer("www.xiaomi.com你好https://abc.baidu.com");
        System.out.println(textTokenizer);
    }
}
